package org.bdware.sc.crdt;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Objects;

public class SharableVarState {
    private String myId;

    private SharableVarConfiguration sharableVarConfiguration;

    private int myIndex;

    private boolean readerFlag;

    private boolean writerFlag;
    private List<String> sendTo;

    private long interval;

    public SharableVarState(String myId, SharableVarConfiguration sharableVarConfiguration) {
        this.myId = myId;
        this.sharableVarConfiguration = sharableVarConfiguration;
        this.myIndex = parseMyIndex();
        this.sendTo = parseSendTo();
        this.interval = sharableVarConfiguration.interval[this.myIndex];
        for (int readerIndex : this.sharableVarConfiguration.readerIndexes) {
            if (Objects.equals(readerIndex, myIndex)) {
                this.readerFlag = true;
                break;
            }
        }
        for (int writerIndex : this.sharableVarConfiguration.writerIndexes) {
            if (Objects.equals(writerIndex, myIndex)) {
                this.writerFlag = true;
                break;
            }
        }
    }

    private int parseMyIndex() {
        for (int i = 0; i < sharableVarConfiguration.getNodeIds().length; i++) {
            if (Objects.equals(sharableVarConfiguration.getNodeIds()[i], myId)) {
                return i;
            }
        }
        return -1;
    }

    private List<String> parseSendTo() {
        if (myIndex < 0 || myIndex >= sharableVarConfiguration.sendTo.length) {
            return new ArrayList<>();
        }
        LinkedHashSet<String> result = new LinkedHashSet<>();
        int[] sendToIndexes = sharableVarConfiguration.sendTo[myIndex];
        for (int sendToIndex : sendToIndexes) {
            if (sendToIndex >= 0 && sendToIndex < sharableVarConfiguration.nodeIds.length
                    && sendToIndex != myIndex) {
                result.add(sharableVarConfiguration.nodeIds[sendToIndex]);
            }
        }
        return new ArrayList<>(result);
    }

    public String getMyId() {
        return myId;
    }

    public void setMyId(String myId) {
        this.myId = myId;
    }

    public SharableVarConfiguration getSharableVarConfiguration() {
        return sharableVarConfiguration;
    }

    public void setSharableVarConfiguration(SharableVarConfiguration sharableVarConfiguration) {
        this.sharableVarConfiguration = sharableVarConfiguration;
    }

    public int getMyIndex() {
        return myIndex;
    }

    public void setMyIndex(int myIndex) {
        this.myIndex = myIndex;
    }

    public boolean isReaderFlag() {
        return readerFlag;
    }

    public void setReaderFlag(boolean readerFlag) {
        this.readerFlag = readerFlag;
    }

    public boolean isWriterFlag() {
        return writerFlag;
    }

    public void setWriterFlag(boolean writerFlag) {
        this.writerFlag = writerFlag;
    }

    public List<String> getSendTo() {
        return sendTo;
    }

    public void setSendTo(List<String> sendTo) {
        this.sendTo = sendTo;
    }

    public long getInterval() {
        return interval;
    }

    public void setInterval(long interval) {
        this.interval = interval;
    }

    public static class SharableVarConfiguration {
        String[] nodeIds;
        int[] writerIndexes;
        int[] readerIndexes;
        int[][] sendTo;
        long[] interval;
        long maxDelay;

        public String[] getNodeIds() {
            return nodeIds;
        }

        public void setNodeIds(String[] nodeIds) {
            this.nodeIds = nodeIds;
        }

        public int[] getWriterIndexes() {
            return writerIndexes;
        }

        public void setWriterIndexes(int[] writerIndexes) {
            this.writerIndexes = writerIndexes;
        }

        public int[] getReaderIndexes() {
            return readerIndexes;
        }

        public void setReaderIndexes(int[] readerIndexes) {
            this.readerIndexes = readerIndexes;
        }

        public int[][] getSendTo() {
            return sendTo;
        }

        public void setSendTo(int[][] sendTo) {
            this.sendTo = sendTo;
        }

        public long[] getInterval() {
            return interval;
        }

        public void setInterval(long[] interval) {
            this.interval = interval;
        }

        public long getMaxDelay() {
            return maxDelay;
        }

        public void setMaxDelay(long maxDelay) {
            this.maxDelay = maxDelay;
        }
    }
}
