package org.bdware.sc.crdt.proxy;

import io.netty.util.HashedWheelTimer;
import io.netty.util.Timeout;
import io.netty.util.TimerTask;
import org.bdware.crdt.basic.JoinableCRDT;
import org.bdware.sc.crdt.SharableVarManager;
import org.bdware.sc.crdt.SharableVarState;
import org.bdware.sc.util.JsonUtil;

import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public abstract class SharableVar<T extends JoinableCRDT> {
    public final HashedWheelTimer HASHED_WHEEL_TIMER = new HashedWheelTimer(r -> {
        Thread t = Executors.defaultThreadFactory().newThread(r);
        t.setDaemon(true);
        return t;
    }, 5, TimeUnit.MILLISECONDS, 2);

    public final Queue<JoinableCRDT> localDeltaQueue = new LinkedList<>();
    public final Queue<JoinableCRDT> remoteDeltaQueue = new LinkedList<>();
    public T self;
    protected String varId;
    private SyncTimerTask syncTimerTask;
    private SharableVarState sharableVarState;

    public SharableVar(String varId, String cpId,
            SharableVarState.SharableVarConfiguration resolveResult) {
        this.varId = varId;
        this.sharableVarState = new SharableVarState(cpId, resolveResult);
        this.HASHED_WHEEL_TIMER.newTimeout(timeout -> {
            syncTimerTask = new SyncTimerTask();
            HASHED_WHEEL_TIMER.newTimeout(syncTimerTask, sharableVarState.getInterval(),
                    TimeUnit.MILLISECONDS);
        }, 0, TimeUnit.MILLISECONDS);
    }

    private void syncVar() {
        if (localDeltaQueue.isEmpty()) {
            return;
        }
        JoinableCRDT joinedDelta = newEmptyDelta();
        synchronized (localDeltaQueue) {
            while (!localDeltaQueue.isEmpty()) {
                JoinableCRDT delta = localDeltaQueue.poll();
                joinedDelta.join(delta);
            }
        }
        String content = JsonUtil.toJson(localDeltaQueue);
        SharableVarManager.instance.broadcastSyncMessage(varId, sharableVarState.getSendTo(),
                content);
    }

    abstract protected T newEmptyDelta();

    class SyncTimerTask implements TimerTask {
        @Override
        public void run(Timeout timeout) throws Exception {
            try {
                syncVar();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                HASHED_WHEEL_TIMER.newTimeout(syncTimerTask, sharableVarState.getInterval(),
                        TimeUnit.MILLISECONDS);
            }
        }
    }
}
